//
// hello.speaker.45.c
//
// speaker PWM hello-world
//    square wave, pseudo-random frequency
//
// Neil Gershenfeld
// 11/14/10
//
// (c) Massachusetts Institute of Technology 2010
// Permission granted for experimental and personal use;
// license for commercial sale available from MIT.
//

#include <avr/io.h>
#include <util/delay.h>

#define output(directions,pin) (directions |= pin) // set port direction for output
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set

#define cycle_delay() _delay_us(2) // cycle delay
#define current 150 // PWM current
#define off 255 // PWM off

#define MOSFET_pin (1 << PB1)
#define MOSFET_port PORTB
#define MOSFET_direction DDRB

// Initialize/declares static variables
   //
   // main
   //
   static uint16_t cycle,cycles,delay,bit,semiquaver;
   static uint32_t number_of_min_note_delay = 250000; // in 1 sec

   static uint16_t A5 = 880; // A5 in Hz
   static uint16_t G5_sharp = 831; // G#5 in Hz
   static uint16_t G5 = 784; // G5 in Hz
   static uint16_t F5 = 698; // F5 in Hz
   static uint16_t E5 = 659; // E5 in Hz
   static uint16_t D5 = 587; // D5 in Hz
   static uint16_t C5 = 523; // C5 in Hz

   static uint16_t B4 = 494; // B4 in Hz
   static uint16_t Bb4 = 466; // Bb4 in Hz
   static uint16_t A4 = 440; // A4 in Hz
   static uint16_t Ab4 = 415; // Ab4 in Hz
   static uint16_t G4 = 392; // G4 in Hz
   static uint16_t F4_sharp = 370; // F#4 in Hz
   static uint16_t F4 = 349; // F4 in Hz
   static uint16_t E4 = 330; // E4 in Hz
   static uint16_t D4 = 294; // D4 in Hz
   static uint16_t Db4 = 277; // Db4 in Hz
   static uint16_t C4 = 262; // C4 in Hz

   static uint16_t B3 = 247; // B3 in Hz
   static uint16_t Bb3 = 233; // Bb3 in Hz
   static uint16_t A3_sharp = 233; // A#3 in Hz
   static uint16_t A3 = 220; // A3 in Hz
   static uint16_t Ab3 = 208; // Ab3 in Hz
   static uint16_t G3 = 196; // G3 in Hz
   static uint16_t F3_sharp = 185; // F#3 in Hz
   static uint16_t F3 = 175; // F3 in Hz
   static uint16_t E3 = 165; // E3 in Hz
   static uint16_t D3 = 147; // D3 in Hz
   static uint16_t Db3= 139; // Db3 in Hz
   static uint16_t C3 = 131; // C3 in Hz

   static uint16_t REST = 0;
   static uint16_t tempo = 120; // in BPM (beats per minute)
   //
   // initialize LFSR
   //
   static uint16_t lfsr = 1;

int play_note(uint16_t note, uint16_t num_of_semiquavers)
{
   //
   // update 11 bit LFSR
   //
   bit  = ((lfsr >> 0) ^ (lfsr >> 9)) & 1;
   lfsr =  (lfsr >> 1) | (bit << 10);
   lfsr = number_of_min_note_delay/note;
   if (note == REST)
      lfsr = 1024;
   //
   // loop over cycles
   //
   cycles = (1 << 15) / lfsr;
   for (semiquaver = 0; semiquaver < num_of_semiquavers; ++semiquaver) {
      for (cycle = 0; cycle < cycles; ++cycle) {
         //
         // set PWM current on and delay
         //
         OCR0B = current;
	 if (note == REST)
            OCR0B = off;
         for (delay = 0; delay < lfsr; ++delay)
            cycle_delay();
         //
         // set PWM current off and delay
         //
         OCR0B = off;
         for (delay = 0; delay < lfsr; ++delay)
            cycle_delay();
      }
   }
}

/* The Twilight Zone*/
int play_tune()
{
  play_note(G5_sharp,1);
  play_note(A5,1);
  play_note(G5_sharp,1);
  play_note(E5,1);
}

int main(void) {

   //
   // set clock divider to /1
   //
   CLKPR = (1 << CLKPCE);
   CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
   //
   //
   //
   TCCR0A = ((1 << COM0B0) | (1 << COM0B1) | (1 << WGM01) | (1 << WGM00)); // set OC0B on compare match and set fast PWM mode, 0xFF TOP
   TCCR0B = (1 << CS00); // set timer 0 prescalar to 1
   //
   // initialize MOSFET pin
   //
   clear(MOSFET_port, MOSFET_pin);
   output(MOSFET_direction, MOSFET_pin);

   //
   // main loop
   //
   while (1) {
      play_tune();
      play_tune();
      play_note(REST,16);
   }
}
